.carousel-slider-iframe{width:100%!important}.carousel-slider-iframe-container{position:relative}.carousel-slider-iframe-overlay{bottom:0;left:0;position:absolute;right:0;top:0}.carousel-slider-editor-controls{padding-right:1rem;width:100%}.carousel-slider-editor-controls__logo{float:left;margin-bottom:10px;margin-right:15px;max-height:50px;max-width:50px!important}.carousel-slider-editor-controls__input{padding-bottom:1rem}.carousel-slider-inspector-controls{padding-left:1rem;padding-right:1rem}