(()=>{"use strict";if(window.carouselSliderCssUrl){if(!document.querySelector("#carousel-slider-frontend-css")){let e=((e,t={},r=[])=>{let l=document.createElement(e);return Object.keys(t).length&&Object.entries(t).forEach((([e,t])=>{l.setAttribute(e,t)})),r.length&&l.append(...r),l})("link",{id:"carousel-slider-frontend-css",rel:"stylesheet",media:"all",href:window.carouselSliderCssUrl});document.head.append(e)}}})();