(()=>{"use strict";var e={n:r=>{var t=r&&r.__esModule?()=>r.default:()=>r;return e.d(t,{a:t}),t},d:(r,t)=>{for(var i in t)e.o(t,i)&&!e.o(r,i)&&Object.defineProperty(r,i,{enumerable:!0,get:t[i]})},o:(e,r)=>Object.prototype.hasOwnProperty.call(e,r)};(()=>{const r=React;var t=e.n(r);const i=jQuery;var a,n,s,l=e.n(i);class o extends r.Component{render(){const e=this.props.slider_id;if(!e)return null;let r=new URL(window.csDivi.site_url);r.searchParams.append("carousel_slider_preview","1"),r.searchParams.append("carousel_slider_iframe","1"),r.searchParams.append("slider_id",e);let i=r.toString();return t().createElement("div",{className:"carousel-slider-iframe-container"},t().createElement("div",{className:"carousel-slider-iframe-overlay"}),t().createElement("iframe",{className:"carousel-slider-iframe",scrolling:"no",src:i,height:"0",width:"500"}))}}a=o,s="carousel_slider_divi_module",(n=function(e){var r=function(e,r){if("object"!=typeof e||!e)return e;var t=e[Symbol.toPrimitive];if(void 0!==t){var i=t.call(e,r||"default");if("object"!=typeof i)return i;throw new TypeError("@@toPrimitive must return a primitive value.")}return("string"===r?String:Number)(e)}(e,"string");return"symbol"==typeof r?r:r+""}(n="slug"))in a?Object.defineProperty(a,n,{value:s,enumerable:!0,configurable:!0,writable:!0}):a[n]=s,l()(window).on("et_builder_api_ready",((e,r)=>{r.registerModules([o])}))})()})();